#!/usr/bin/env python
import os
import sys
import subprocess
import socket
import re
import signal
import errno
from tracerCommon import RenderParameterFields

g_messageId = 512
g_clientSocket = None
g_bNoParams = False


def Message(senderID, receiverID, timestamp, message, messageData):
    if not g_bNoParams:
        message = RenderParameterFields(message, ",".join(messageData))
    global g_messageId
    newId = '0x%02x' % g_messageId
    g_messageId += 1
    s = 'messageSent| -t%s| %s| %s| %s|\n' % \
        (timestamp, senderID, newId, message)
    g_clientSocket.send(s)
    s = 'messageReceived| -t%s| %s| %s| %s|\n' % \
        (timestamp, receiverID, newId, message)
    g_clientSocket.send(s)


def main():
    if "-noParams" in sys.argv:
        global g_bNoParams
        g_bNoParams = True
        sys.argv.remove("-noParams")
    if len(sys.argv) != 4:
        print "Usage:", sys.argv[0], "[-noParams] <ipaddress> <port> <application>"
        print "where ipaddress and port point to the tracerd port."
        sys.exit(1)
    try:
        ipaddress, port = sys.argv[1:3]
        port = int(port)
        global g_clientSocket
        g_clientSocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        g_clientSocket.connect((ipaddress, port))
    except:
        print "Could not connect to tracerd..."
        sys.exit(1)

    os.putenv("TASTE_INNER_MSC", "1")
    os.putenv("ASSERT_IGNORE_GUI_ERRORS", "1")

    p = None
    try:
        p = subprocess.Popen(sys.argv[3], stdout=subprocess.PIPE)
        messageData = {}
        tasks = {}
        for line in iter(p.stdout.readline, ''):
            lline = line.strip()
            # print "WORKING ON:", lline
            m = re.match('^INNERDATA: ([^:]*)::(.*)$', lline)
            if m:
                messageData.setdefault(m.group(1), []).append(m.group(2))
            elif lline.startswith('INNER: '):
                sender, receiver, ri, timestamp = lline[7:].split(',')
                # print sender, receiver, ri, timestamp
                if ri not in messageData.keys():
                    messageData[ri] = []
                for task in [sender, receiver]:
                    if task not in tasks:
                        newTaskId = '0x%02x' % (1+ len(tasks))
                        tasks[task] = newTaskId
                        g_clientSocket.send('taskCreated| -n%s| %s|\n' % (task, newTaskId))
                Message(tasks[sender], tasks[receiver], timestamp, ri, messageData[ri])
                messageData[ri] = []
            else:
                sys.stdout.write(line)
                sys.stdout.flush()
    except KeyboardInterrupt:
        if p:
            print "Sending SIGINT to", sys.argv[3]
            p.send_signal(signal.SIGINT)
    except socket.error, e:
        if e.args[0] == errno.EINTR:  # Ctrl-C interrupted socket system call
            if p:
                print "Sending SIGINT to", sys.argv[3]
                p.send_signal(signal.SIGINT)
        else:
            if p:
                print "Error while speaking to tracerd:", e.args[1]
                p.kill()
    except:
        print "Unexpected error:", sys.exc_info()[0]
        if p:
            print "Killing", sys.argv[3]
            p.kill()
    if p:
        p.wait()
    print "Clean shutdown"

if __name__ == "__main__":
    main()
