#
# (C) Semantix Information Technologies.
#
# Semantix Information Technologies is licensing the code of the
# Data Modelling Tools (DMT) in the following dual-license mode:
#
# Commercial Developer License:
#       The DMT Commercial Developer License is the suggested version
# to use for the development of proprietary and/or commercial software.
# This version is for developers/companies who do not want to comply
# with the terms of the GNU Lesser General Public License version 2.1.
#
# GNU LGPL v. 2.1:
#       This version of DMT is the one to use for the development of
# applications, when you are willing to comply with the terms of the
# GNU Lesser General Public License version 2.1.
#
# Note that in both cases, there are no charges (royalties) for the
# generated code.
#
import re
import os
import commonPy
import commonPy.asnParser
from commonPy.utility import panic, inform
from commonPy.asnAST import AsnBool, AsnInt, AsnReal, AsnString, isSequenceVariable, AsnEnumerated, AsnSequence, AsnSet, AsnChoice, AsnMetaMember, AsnSequenceOf, AsnSetOf, AsnBasicNode
import commonPy.cleanupNodes

# The Python file written to
g_outputFile = None

# The SETers and GETers files
g_outputGetSetH = None
g_outputGetSetC = None

g_bHasStartupRunOnce = False


def Version():
    print "Code generator: " + "$Id: python_A_mapper.py 2400 2012-09-04 10:40:19Z ttsiodras $"  # pragma: no cover


def CleanNameAsPythonWants(name):
    return re.sub(r'[^a-zA-Z0-9_]', '_', name)


def OnStartup(unused_modelingLanguage, asnFile, outputDir):
    os.system("cp \"" + asnFile + "\" \"" + outputDir + "\"")
    if os.getenv("DMT") is None:
        panic("You must set the DMT environment variable")  # pragma: no cover
    os.system("cp \"$DMT/asn2dataModel/Stubs.py\" \"" + outputDir + "\"")
    global g_bHasStartupRunOnce
    if g_bHasStartupRunOnce:
        # Don't rerun, it has already done all the work
        # for all the ASN.1 files used
        return  # pragma: no cover
    else:
        g_bHasStartupRunOnce = True
    if not asnFile.endswith(".asn"):
        panic("The ASN.1 grammar file (%s) doesn't end in .asn" % asnFile)  # pragma: no cover
    outputFilename = os.path.basename(asnFile)
    outputFilename = re.sub(r'[^a-zA-Z0-9_]', '_', outputFilename) + ".py"
    origGrammarBase = os.path.basename(asnFile.replace(".asn", ""))
    base = re.sub(r'[^a-zA-Z0-9_]', '_', origGrammarBase)
    inform("Python_A_mapper: Creating file '%s'...", outputFilename)
    global g_outputFile
    g_outputFile = open(outputDir + outputFilename, 'w')
    g_outputFile.write("import DV\n\n")
    g_outputFile.write("from Stubs import myassert, panicWithCallStack, Clean, DataStream, COMMON\n\n")
    global g_outputGetSetH
    g_outputGetSetH = open(outputDir + base + "_getset.h", "w")
    g_outputGetSetH.write('#ifndef __GETSET_H__\n#define __GETSET_H__\n\n')
    g_outputGetSetH.write('#include "%s.h"\n' % origGrammarBase)
    global g_outputGetSetC
    g_outputGetSetC = open(outputDir + "%s_getset.c"%base, "w")
    g_outputGetSetC.write('#include "%s_getset.h"\n\n'%base)
    Makefile = open(outputDir + "Makefile.python", 'w')

    # Note that this Makefile will use a custom ASN1SCC invocation
    # where "-equal" is passed - the _Equal functions will be generated
    # and used during comparisons of incoming TMs (For MSCs)

    Makefile.write('''
ifeq ($(DMT),)
$(error You must set the DMT environment variable)
endif

ifeq ($(ASN2DATAMODEL),)
ASN2DATAMODEL:=$(DMT)/asn2dataModel/asn2dataModel.py
endif
ifeq ($(ASN1SCC),)
ASN1SCC:=$(DMT)/asn1scc/asn1.exe
endif

PREFIX:=`taste-config --prefix`/share/

GRAMMAR := %(origGrammarBase)s
BASEGRAMMAR := %(base)s
BUILDDIR:= .
OBJ     := $(BUILDDIR)/$(GRAMMAR).o $(BUILDDIR)/asn1crt.o $(BUILDDIR)/real.o $(BUILDDIR)/acn.o $(BUILDDIR)/ber.o $(BUILDDIR)/xer.o $(BUILDDIR)/DV_wrap.o $(BUILDDIR)/$(BASEGRAMMAR)_getset.o

all:    _DV.so 
# ExtraSwigGate

$(BUILDDIR)/DV.i $(BUILDDIR)/$(GRAMMAR)_getset.c:       $(GRAMMAR).asn
	mkdir -p $(BUILDDIR)
	$(ASN2DATAMODEL) -toPython -o $(BUILDDIR) $<

$(BUILDDIR)/asn1crt.c $(BUILDDIR)/$(GRAMMAR).c $(BUILDDIR)/real.c $(BUILDDIR)/acn.c $(BUILDDIR)/ber.c $(BUILDDIR)/xer.c $(BUILDDIR)/$(GRAMMAR).h $(BUILDDIR)/asn1crt.h:       $(GRAMMAR).asn
	if [ ! -f "$(GRAMMAR).acn" ] ; then mono $(ASN1SCC) -ACND -o $(BUILDDIR) $< ; fi
	mono $(ASN1SCC) -ACN -c -uPER -equal -wordSize 8 -o $(BUILDDIR) $< $(GRAMMAR).acn

$(BUILDDIR)/DV_wrap.c:  $(BUILDDIR)/DV.i        $(BUILDDIR)/$(GRAMMAR).h
	swig -includeall -outdir $(BUILDDIR) -python $(BUILDDIR)/DV.i

ExtraSwigGate:
	cp $(PREFIX)/asn1-editor/InterfaceEnum.i .
	swig -includeall -outdir . -python ./InterfaceEnum.i && gcc -g -shared  -fPIC `python-config --includes` -o _InterfaceEnum.so InterfaceEnum_wrap.c

$(BUILDDIR)/%%.o:       $(BUILDDIR)/%%.c
	gcc -g -fPIC -c `python-config --includes` -o $@ $<

_DV.so: ${OBJ}
	gcc -g -fPIC -shared `python-config --ldflags` -o $@ $^

clean:
	rm -f $(BUILDDIR)/asn1crt.?  $(BUILDDIR)/real.?  $(BUILDDIR)/$(GRAMMAR).?  $(BUILDDIR)/acn.?  $(BUILDDIR)/ber.? $(BUILDDIR)/xer.?
	rm -f $(BUILDDIR)/DV.py $(BUILDDIR)/*.pyc $(BUILDDIR)/DV_wrap.? _DV.so
	rm -f $(BUILDDIR)/$(GRAMMAR)_getset.? $(BUILDDIR)/$(GRAMMAR)_asn.py $(BUILDDIR)/DV.i
''' % {'base': base, 'origGrammarBase': origGrammarBase})
    Makefile.close()
    DV_file = open(outputDir + "DV.i", 'w')
    DV_file.write('''%%module DV

%%{
#include "%(origGrammarBase)s.h"
#include "%(base)s_getset.h"
%%}

#include "%(origGrammarBase)s.h"
#include "%(base)s_getset.h"

%%include "carrays.i"
%%array_functions(byte,byte_SWIG_PTR)
%%array_functions(int,int_SWIG_PTR)
%%array_functions(double,double_SWIG_PTR)
''' % {'base': base, 'origGrammarBase': origGrammarBase})
    for nodeTypename, node in commonPy.asnParser.g_names.iteritems():
        if node._isArtificial:
            continue
        if nodeTypename not in ["byte", "int", "double"]:  # already done these
            DV_file.write(
                '%%array_functions(%s,%s_SWIG_PTR)\n' %
                (CleanNameAsPythonWants(nodeTypename), CleanNameAsPythonWants(nodeTypename)))

    DV_file.write('''
''')
    DV_file.close()
    CreateDeclarationsForAllTypes(commonPy.asnParser.g_names, commonPy.asnParser.g_leafTypeDict)
    g_outputGetSetH.write('\n#endif\n')


def OnBasic(nodeTypename, node, leafTypeDict):
    pass


def OnSequence(nodeTypename, node, leafTypeDict):
    pass


def OnSet(nodeTypename, node, leafTypeDict):
    pass


def OnEnumerated(nodeTypename, node, leafTypeDict):
    pass


def OnSequenceOf(nodeTypename, node, leafTypeDict):
    pass


def OnSetOf(nodeTypename, node, leafTypeDict):
    pass


def OnChoice(nodeTypename, node, leafTypeDict):
    pass


def OnShutdown():
    pass


class Params:
    cTypes = {
        "BOOLEAN": "flag",
        "INTEGER": "asn1SccSint",
        "REAL": "double",
        "ENUMERATED": "int",
        "OCTET STRING": "byte*",
    }

    def __init__(self, nodeTypename):
        self._vars = []
        self._types = []
        self._nodeTypeName = nodeTypename

    def AddParam(self, node, varName, unused_leafTypeDict):
        # Handle variable name
        while varName in self._vars:
            varName += "_"
        # Handle type
        # special case:int is used for SEQUENCE_OF (iDx)
        assert node == "int"
        self._vars.append(varName)
        self._types.append("int")
        return True
        # For others, lookup the C type
        #try:
        #    realLeafType = leafTypeDict[node._leafType]
        #except:
        #    #panic("Python_A_mapper: Only primitive types can be C params, not %s" % node.Location())
        #    # the lookup will fail for non-primitives, which we ignore
        #    return False

        #self._vars.append(varName)
        #try:
        #    self._types.append(self.cTypes[realLeafType])
        #except:
        #    panic("Python_A_mapper: Can't map (%s,%s) to C type\n" % (varName, realLeafType))
        #return True

    def Pop(self):
        self._vars.pop()
        self._types.pop()

    def GetDecl(self):
        params = CleanNameAsPythonWants(self._nodeTypeName) + "* root"
        for v, t in zip(self._vars, self._types):
            params += ", "+t+" "+v
        return params


def CommonBaseImpl(comment, ctype, path, params, accessPathInC, postfix=""):
    g_outputGetSetH.write("\n/* %s */\n%s %s_Get%s(%s);\n" % (comment, ctype, path, postfix, params.GetDecl()))
    g_outputGetSetC.write("\n/* %s */\n%s %s_Get%s(%s)\n" % (comment, ctype, path, postfix, params.GetDecl()))
    g_outputGetSetC.write("{\n")
    g_outputGetSetC.write("    return (*root)" + accessPathInC + ";\n")
    g_outputGetSetC.write("}\n")
    g_outputGetSetH.write("\n/* %s */\nvoid %s_Set%s(%s, %s value);\n" % (comment, path, postfix, params.GetDecl(), ctype))
    g_outputGetSetC.write("\n/* %s */\nvoid %s_Set%s(%s, %s value)\n" % (comment, path, postfix, params.GetDecl(), ctype))
    g_outputGetSetC.write("{\n")
    g_outputGetSetC.write("    (*root)" + accessPathInC + " = value;\n")
    g_outputGetSetC.write("}\n")


# def CommonBaseImplSequenceFixed(comment, ctype, path, params, accessPathInC, node, postfix = ""):
def CommonBaseImplSequenceFixed(comment, ctype, path, params, _, node, postfix=""):
    g_outputGetSetH.write("\n/* %s */\n%s %s_Get%s(%s);\n" % (comment, ctype, path, postfix, params.GetDecl()))
    g_outputGetSetC.write("\n/* %s */\n%s %s_Get%s(%s)\n" % (comment, ctype, path, postfix, params.GetDecl()))
    g_outputGetSetC.write("{\n")
    g_outputGetSetC.write("    return " + str(node._range[-1]) + ";\n")
    g_outputGetSetC.write("}\n")
    g_outputGetSetH.write("\n/* %s */\nvoid %s_Set%s(%s, %s value);\n" % (comment, path, postfix, params.GetDecl(), ctype))
    g_outputGetSetC.write("\n/* %s */\nvoid %s_Set%s(%s, %s value)\n" % (comment, path, postfix, params.GetDecl(), ctype))
    g_outputGetSetC.write("{\n")
    g_outputGetSetC.write("    puts(\"WARNING: setting length of fixed-length sequence\");\n")
    g_outputGetSetC.write("}\n")


def CreateGettersAndSetters(path, params, accessPathInC, node, names, leafTypeDict):
    if isinstance(node, str):
        node = names[node]
    if isinstance(node, AsnMetaMember):
        node = names[node._containedType]

    if isinstance(node, AsnBool):
        CommonBaseImpl("BOOLEAN", "flag", path, params, accessPathInC)
    elif isinstance(node, AsnInt):
        CommonBaseImpl("INTEGER", "asn1SccSint", path, params, accessPathInC)
    elif isinstance(node, AsnReal):
        CommonBaseImpl("REAL", "double", path, params, accessPathInC)
    elif isinstance(node, AsnString):
        if node._range == []:
            panic("Python_A_mapper: string (in %s) must have a SIZE constraint!\n" % node.Location())  # pragma: no cover
        if isSequenceVariable(node):
            CommonBaseImpl("OCTETSTRING", "long", path, params, accessPathInC+".nCount", "Length")
        else:
            CommonBaseImplSequenceFixed("OCTETSTRING", "long", path, params, accessPathInC+".nCount", node, "Length")
        params.AddParam('int', "iDx", leafTypeDict)
        CommonBaseImpl("OCTETSTRING_bytes", "byte", path+"_iDx", params, accessPathInC + (".arr["+params._vars[-1]+"]"), "")
        params.Pop()
    elif isinstance(node, AsnEnumerated):
        CommonBaseImpl("ENUMERATED", "int", path, params, accessPathInC)
    elif isinstance(node, AsnSequence) or isinstance(node, AsnSet) or isinstance(node, AsnChoice):
        if isinstance(node, AsnChoice):
            CommonBaseImpl("CHOICE selector", "int", path+"_kind", params, accessPathInC+".kind")
        for child in node._members:
            childNode = child[1]
            childVarname = CleanNameAsPythonWants(child[0])
            if isinstance(childNode, AsnMetaMember):
                childNode = names[childNode._containedType]
            union = ""
            if isinstance(node, AsnChoice):
                union = ".u"
            CreateGettersAndSetters(path+"_"+childVarname, params, accessPathInC+union+"."+childVarname, child[1], names, leafTypeDict)
    elif isinstance(node, AsnSequenceOf) or isinstance(node, AsnSetOf):
        containedNode = node._containedType
        if isinstance(containedNode, str):
            containedNode = names[containedNode]
        if isSequenceVariable(node):
            CommonBaseImpl("SEQUENCEOF/SETOF", "long", path, params, accessPathInC+".nCount", "Length")
        else:
            CommonBaseImplSequenceFixed("SEQUENCEOF/SETOF", "long", path, params, accessPathInC+".nCount", node, "Length")
        params.AddParam('int', "iDx", leafTypeDict)
        CreateGettersAndSetters(path+"_iDx", params, accessPathInC + (".arr["+params._vars[-1]+"]"), node._containedType, names, leafTypeDict)
        params.Pop()


def DumpTypeDumper(codeIndent, outputIndent, lines, variableName, node, names):
    if isinstance(node, AsnBool):
        lines.append(codeIndent + 'print "%s"+str(%s.Get()!=0)' % (outputIndent, variableName))
    elif isinstance(node, AsnInt):
        lines.append(codeIndent + 'print "%s"+str(%s.Get())' % (outputIndent, variableName))
    elif isinstance(node, AsnReal):
        lines.append(codeIndent + 'print "%s"+str(%s.Get())' % (outputIndent, variableName))
    elif isinstance(node, AsnString):
        lines.append(codeIndent + 'print "%s"+str(%s.GetPyString())' % (outputIndent, variableName))
    elif isinstance(node, AsnEnumerated):
        lines.append(codeIndent + 'print "%s"+str(%s.Get())' % (outputIndent, variableName))
    elif isinstance(node, AsnChoice) or isinstance(node, AsnSet) or isinstance(node, AsnSequence):
        extraIndent = ""
        if isinstance(node, AsnChoice):
            extraIndent = "    "
        for child in node._members:
            if isinstance(node, AsnChoice):
                lines.append(codeIndent + 'if %s.kind.Get() == DV.%s:' % (variableName, CleanNameAsPythonWants(child[2])))
            lines.append(codeIndent + extraIndent + 'print "%s%s:"' % (outputIndent, child[0]))
            childNode = child[1]
            if isinstance(childNode, AsnMetaMember):
                childNode = names[childNode._containedType]
            DumpTypeDumper(codeIndent + extraIndent, outputIndent +"    ", lines, variableName+"."+CleanNameAsPythonWants(child[0]), childNode, names)
    elif isinstance(node, AsnSetOf) or isinstance(node, AsnSequenceOf):
        containedNode = node._containedType
        if isinstance(containedNode, str):
            containedNode = names[containedNode]
        for i in xrange(0, node._range[-1]):
            lines.append(codeIndent + 'if %s.GetLength()>%d:' % (variableName, i))
            lines.append(codeIndent + '    print "%s[%d]:"' % (outputIndent, i))
            DumpTypeDumper(codeIndent+"    ", outputIndent+"    ", lines, variableName+'['+str(i)+']', containedNode, names)


def CreateDeclarationForType(nodeTypename, names, leafTypeDict):
    node = names[nodeTypename]
    name = CleanNameAsPythonWants(nodeTypename)
    if isinstance(node, AsnBasicNode) or isinstance(node, AsnEnumerated) or \
        isinstance(node, AsnSequence) or isinstance(node, AsnSet) or \
        isinstance(node, AsnChoice) or isinstance(node, AsnSequenceOf) or \
        isinstance(node, AsnSetOf):

        g_outputFile.write("class " + name + "(COMMON):\n")
        if isinstance(node, AsnEnumerated):
            g_outputFile.write("    # Allowed enumerants:\n")
            allowed = "    allowed = ["
            for member in node._members:
                # member[0] enumerant name, member[1] integer value (or None)
                if member[1] is not None:
                    g_outputFile.write("    %s = %s\n" % (CleanNameAsPythonWants(member[0]), member[1]))
                    allowed += ("%s, " % (CleanNameAsPythonWants(member[0])))
                else:  # pragma: no cover
                    panic("Python_A_mapper: must have values for enumerants (%s)" % node.Location())  # pragma: no cover
            g_outputFile.write(allowed + "]\n")
        g_outputFile.write("    def __init__(self):\n")
        g_outputFile.write("        COMMON.__init__(self, \"" + name + "\")\n")
        if isinstance(node, AsnString):
            g_outputFile.write('''#\n''')
        CreateGettersAndSetters(name+"_", Params(nodeTypename), "", node, names, leafTypeDict)
        g_outputFile.write("    def PrintAll(self):\n")
        lines = []
        DumpTypeDumper("        ", "", lines, "self", names[nodeTypename], names)
        g_outputFile.write("\n".join(lines) + "\n\n")

    else:  # pragma: no cover
        panic("Unexpected ASN.1 type... Send this grammar to Semantix")  # pragma: no cover


def CreateDeclarationsForAllTypes(names, leafTypeDict):
    for nodeTypename in names:
        if not names[nodeTypename]._isArtificial and not commonPy.cleanupNodes.IsBadType(nodeTypename):
            CreateDeclarationForType(nodeTypename, names, leafTypeDict)
