----------------------------------------------------------------------------------------------
-- Files       : customip0.vhd          
-- Version     : 0.1                        
-- Date        : 07/06/2010                 
-- Author      : Yann LECLERC - M3Systems   
----------------------------------------------------------------------------------------------
-- Description :
-- Example of clocked custom IP, with asynchronous reset (low acknowledgment)
-- This IP takes two 32 bits variables as inputs and drives two 32 bits variables as outputs.
-- When start_compute is high, it copies its two inputs to its corresponding outputs, and sets
-- finish_compute to high, as long as start_compute is high. 
----------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity customip0 is
	port(
		--Input signals--
		custom_var_in_1 : in std_logic_vector(31 downto 0);
		custom_var_in_2 : in std_logic_vector(31 downto 0);
		
		--Output signals--
		custom_var_out_1 : out std_logic_vector(31 downto 0);
		custom_var_out_2 : out std_logic_vector(31 downto 0);
		
		--Control signals--
		start_compute : in std_logic;
		finish_compute : out std_logic;
		
		rst : in std_logic;
		
		clk_compute : in std_logic
		);
end customip0;

architecture archicustomip0 of customip0 is
	
begin
	process(clk_compute,rst)
		variable run : std_logic;
	begin
		if rst='0' then -- Asynchronous reset
		
			finish_compute <= '0';
			custom_var_out_1 <= x"00000000";
			custom_var_out_2 <= x"00000000";
			run := '1';
			
		elsif clk_compute'event and clk_compute='1' then
		
			if start_compute = '0' then
				finish_compute <= '0';
				run := '1';
			elsif run = '1' then 
			
				custom_var_out_1 <= custom_var_in_1;
				custom_var_out_2 <= custom_var_in_2;
				
				run := '0';
				finish_compute <= '1';
			end if;
		end if;
	end process;
end archicustomip0;

