-----------------------------------------------------------------------------
--
-- THIS FILE IS GENERATED AUTOMATICALLY BY TASTE - DON'T MODIFY IT!!!
--
-----------------------------------------------------------------------------
--  LEON3 Demonstration design
--  Copyright (C) 2004 Jiri Gaisler, Gaisler Research
------------------------------------------------------------------------------
--  This file is a part of the GRLIB VHDL IP LIBRARY
--  Copyright (C) 2003 - 2008, Gaisler Research
--  Copyright (C) 2008 - 2010, Aeroflex Gaisler
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 2 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Files       : architecture_top.vhd          
-- Version     : 0.1                        
-- Date        : 11/06/2010                 
-- Mofified by : Yann LECLERC - M3Systems   
------------------------------------------------------------------------------
-- Description : This file is based on LEON3 Demonstration design. 
-- It implements the different components of the architecture : 
-- Gailser reset generator, clock generator, clock pads and PCI pads. 
-- Gailser simple target PCI
-- Gailser AHB controller
-- Gailser APB bridge
-- APB wrappers
--
-- The index mapping on the AHB/APB bus is as follow : 
-- Simple target PCI : AHB master 0
-- APB Bridge : AHB slave 0
-- APB wrappers : APB slave 0 to 1
--
-- The address mapping on the AHB/APB bus is as follow :
-- APB bridge : 0x80000000-0x80100000
-- APB wrappers: 256 bytes contiguous address space, mapped into APB bridge 
-- address space from 0x80000000, thus:
-- APB wrapper 0 : 0x80000000-0x800000ff
-- APB wrapper 1 : 0x80000100-0x800001ff
--
-- The index of a wrapper on the APB bus, is defined throught the pindex 
-- generic, its address throught the paddr generic (for example paddr=1 means
-- that wrapper address space is from 0x80000100 to 0x800001ff). The apbo port
-- of the wrapper is linked with apbo(index) signal.
--
-- In order to instanciate a new wrapper, the following constraints must be
-- met :
-- Maximum number of APB wrapper is 16
-- Index of the new wrapper must be different than those of the others
-- Address space of the new wrapper must not overlap those of the others 
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library grlib, techmap;
use grlib.amba.all;
use grlib.stdlib.all;
use techmap.gencomp.all;

library gaisler;
use gaisler.misc.all;
use gaisler.pci.all;

library m3s;
use m3s.apbwrapper_declaration.all;

use work.config.all;

entity TASTE_hardware_architecture is
  generic (
    fabtech   : integer := CFG_FABTECH;
    memtech   : integer := CFG_MEMTECH;
    padtech   : integer := CFG_PADTECH;
    clktech   : integer := CFG_CLKTECH
  );
  port (
    resetn	: in  std_logic;
    clk		: in  std_logic;
    pllref 	: in  std_logic; 

    pci_rst     : inout std_logic;		
    pci_clk 	: in std_logic;
    pci_gnt     : in std_logic;
    pci_idsel   : in std_logic; 
    pci_lock    : inout std_logic;
    pci_ad 	: inout std_logic_vector(31 downto 0);
    pci_cbe 	: inout std_logic_vector(3 downto 0);
    pci_frame   : inout std_logic;
    pci_irdy 	: inout std_logic;
    pci_trdy 	: inout std_logic;
    pci_devsel  : inout std_logic;
    pci_stop 	: inout std_logic;
    pci_perr 	: inout std_logic;
    pci_par 	: inout std_logic;    
    pci_req 	: inout std_logic;
    pci_serr    : inout std_logic;
    pci_host   	: in std_logic;
    pci_66	: in std_logic

	);
end;

architecture rtl of TASTE_hardware_architecture is

signal apbi  : apb_slv_in_type;
signal apbo  : apb_slv_out_vector := (others => apb_none);
signal ahbsi : ahb_slv_in_type;
signal ahbso : ahb_slv_out_vector := (others => ahbs_none);
signal ahbmi : ahb_mst_in_type;
signal ahbmo : ahb_mst_out_vector := (others => ahbm_none);

signal clkm, rstn, rstraw, pciclk, lclk, pci_lclk : std_logic;
signal cgi   : clkgen_in_type;
signal cgo   : clkgen_out_type;

signal pcii : pci_in_type;
signal pcio : pci_out_type;

constant BOARD_FREQ : integer := 50000;	-- Board frequency in KHz


begin

----------------------------------------------------------------------
---  Reset and Clock generation  -------------------------------------
----------------------------------------------------------------------

	cgi.pllctrl <= "00"; cgi.pllrst <= rstraw;

	pllref_pad : clkpad 
	generic map (tech => padtech) 
	port map (pllref, cgi.pllref); 
	
	clk_pad : clkpad 
	generic map (tech => padtech) 
	port map (clk, lclk); 
	
	pci_clk_pad : clkpad 
	generic map (tech => padtech, level => pci33) 
	port map (pci_clk, pci_lclk); 
	
	clkgen0 : clkgen  		
    generic map (clktech, CFG_CLKMUL, CFG_CLKDIV, 0, 
				0, CFG_PCI, CFG_PCIDLL, CFG_PCISYSCLK, BOARD_FREQ)
    port map (lclk, pci_lclk, clkm, open, open, open, pciclk, cgi, cgo);

	rst0 : rstgen			
	port map (resetn, clkm, cgo.clklock, rstn, rstraw);

----------------------------------------------------------------------
---  AHB CONTROLLER --------------------------------------------------
----------------------------------------------------------------------

	ahb0 : ahbctrl 		
	generic map (defmast => CFG_DEFMST, split => CFG_SPLIT, 
				rrobin => CFG_RROBIN, ioaddr => CFG_AHBIO,
				ioen => 1,nahbm => 2, nahbs => 1)
	port map (rstn, clkm, ahbmi, ahbmo, ahbsi, ahbso);

-----------------------------------------------------------------------
---  PCI Simple Target-------------------------------------------------
-----------------------------------------------------------------------

	pci0 : pci_target 
	generic map (hindex => 0,device_id => CFG_PCIDID, vendor_id => CFG_PCIVID)
	port map (rstn, clkm, pciclk, pcii, pcio, ahbmi, ahbmo(0));


	pcipads0 : pcipads 
	generic map (padtech => padtech)	
	port map ( pci_rst, pci_gnt, pci_idsel, pci_lock, pci_ad, pci_cbe,
			pci_frame, pci_irdy, pci_trdy, pci_devsel, pci_stop, pci_perr,
			pci_par, pci_req, pci_serr, pci_host, pci_66, pcii, pcio );
 
----------------------------------------------------------------------
---  APB Bridge  -----------------------------------------------------
----------------------------------------------------------------------

	apb0 : apbctrl				
	generic map (hindex => 0, haddr => CFG_APBADDR)
	port map (rstn, clkm, ahbsi, ahbso(0), apbi, apbo );

-----------------------------------------------------------------------
---  Custom IP wrapper ------------------------------------------------
-----------------------------------------------------------------------

	custom0 : apbwrapper0			
	generic map (pindex => 0, paddr => 0)
	port map (apbi, apbo(0), rstn,clkm);

	custom1 : apbwrapper1			
	generic map (pindex => 1, paddr => 1)
	port map (apbi, apbo(1), rstn,clkm);

	custom2 : apbwrapper2			
	generic map (pindex => 2, paddr => 2)
	port map (apbi, apbo(2), rstn,clkm);

end;
